/* ========================================
 *

 *
 * ========================================
*/

#include <RMS.h>
#include <math.h>       // sqrt
#include <FastSqrt.h>   // fsqrt(): fast square root calculator 

#define sqrt2 1.4142135623730950488016887242097         //sqrt(2)

//static double SCALE_16BIT = 32768.0f;     // ADC half-scale at 16-bit
static double RMS_TO_V = 1.024 / 32768.0;   // conversion coeff RMS_d (RMS16) to Volts

static uint32 Wght32;                       // Total Gaussian weight (sum of weighting coefficients
static uint8 BufferSection = 0;             // ..0,..1,..2,..3
                    


//==============================================================================
// Prepare window function and calculate total weight
// Hanning window:
// Hanning = a0 + a1 * Cos (2*PI*i/N); -N/2 <= i <= N/2
// a0 = 0.5 and a1 = 0.5
//==============================================================================

void ConfigRMS16() 
{
    //const double a0 = 0.53836;            // Hamming  (standard deviation (0.5-0.7)E-4)
    //const double a1 = 0.46164;
    const double a0 = 0.5;                  // Hanning  (standard deviation (0.3-0.4)E-4)
    const double a1 = 0.5;
    const double scale = 255.0;             //  8-bit window
    uint16 i=0;
    
    Wght32 = 0;                             // Total window weight  
    
    // Hanning 8-bit-> 
    for( i=0; i<NO_SAMPLES; i++ )           // calculate  1/2 window weight
    {
        double Y = a0 + a1 * cos( M_PI * ((double) i) / ((double) (NO_SAMPLES-1)) ); 
        //uint8 Y8 = (uint8) (scale * Y);
        uint8 Y8 = round (scale * Y);       // convert to 8-bit
        Window[i] = Y8;                     // fill weighting function buffer
        Wght32  += Y8;                      // calculate 1/2 window total weight
    }
    Wght32 <<=1;                            // total window weight 
}


//==============================================================================
// Calculate RMS in half-buffer witn 16-bit precision 
//==============================================================================

double GetRMS16() 
{
    uint64 AccMS64 = 0;                     // Multiply-Accumulate for mean square
    uint16 offset1=0, offset2=0;            // starting offsets for left and right half of Window
    uint16 i;
    int16 v;
  
    
    switch (BufferSection)
    {
        case 0: // -->0..1..2--3
                offset1 = 3 * NO_SAMPLES;
                offset2 = 0;
                BufferSection=1;            // next section
                break;
                
        case 1: // --0-->1..2..3
                offset1 = 0;
                offset2 = NO_SAMPLES;
                BufferSection=2;            // next section
                break;
                
        case 2: // ..0--1-->2..3
                offset1 = NO_SAMPLES;
                offset2 = 2 * NO_SAMPLES;
                BufferSection=3;            // next section
                break;
                
        case 3: // ..0..1--2-->3
                offset1 = 2 * NO_SAMPLES;
                offset2 = 3 * NO_SAMPLES;
                BufferSection=0;            // next section
                break;
    
        default:                            // should not come here
                BufferSection=0;                            
    } 
    
    for( i=0; i<NO_SAMPLES; i++) {                          // calculate RMS
        v = Buffer[i+offset1];                              // Buffer data are signed
        AccMS64 +=  (uint64)(v*v) * Window[(NO_SAMPLES-1)-i]; // accumulate 
    }

    for( i=0; i<NO_SAMPLES; i++) {                          // calculate RMS
        v = Buffer[i+offset2];                              // Buffer data are signed
        AccMS64 +=  (uint64)(v*v) * Window[i];              // accumulate
    }
    

    //RMS16 = sqrt( AccMS64 / GWght32 );         // RMS rounded to 16-bit scale //131us/..us (using standard lib)
    RMS16 = fsqrt( AccMS64 / Wght32 );         // RMS rounded to 16-bit scale //106us/68us  (using fast lib)

    RMS_V = (double)RMS16 * RMS_TO_V;           // calculated RMS in Volts //107us
    
    return RMS_V; 
}

//==============================================================================
//  convert RMSV to VDAC8 code for control output
//==============================================================================

uint8 RMSV_to_VDAC(double value)
{
    const double VDAC_scale_trim = 1.000;                       // VDAC trim factor
    const double VDAC_RANGE = 1.02;                             // VDAC_RMS output full scale (Volts)
    const double Vscale = VDAC_scale_trim * 256.0 / VDAC_RANGE; 
       
    return (uint8) (Vscale * value);     
}

/* [] END OF FILE */


