/* ============================================================================
 * Project Name: RMS_dsADC_wFunc_04a_upload
 *
 * Description:
 *   Demo project showing RMS calculation using window function and quadro-buffering. 
 *   Incoming AC signal is continuously sampled by DeltaSigma ADC (16-bit)
 *   and transferred to the RAM Buffer using DMA.
 *   Once 1/4 Buffer is filled, ISR is fired and Buffer data is processed.   
 *   RMS value calculated using window function (Hanning):
 *   MS = SUM(Buffer^2[i] x Window[i]) / SUM(Window[i])
 *   RMS = sqrt(MS)
 *
 *   Project points
 *   1. ADC is sampling continuously at 4 kHz (1 - 8 kHz)
 *   2. ADC sampling and AC signal are not synchronized
 *   3. DMA transfers ADC data to RAM Buffer (4xN samples, where N=201 (401, 801)
 *   4. ISR fires each 1/4 Buffer is filled to process the data
 *   5. Data processing (RMS) takes approx. 240us (debug) / 120us (release) (N=200) 
 *   6. RMS results update rate is 20 Hz (ADC clock: 4 kHz, N=200) 
 *   7. RMS group delay is 50 ms
 *
 *   Project includes optional arbitrary signal generator, which can be 
 *   safely removed if external signal generator is available.
 *   It requires following custom components:
 *   DDS32_v0_0          : Afbitrary frequency DDS generator. https://community.cypress.com/message/158566#158566
 *   WaveGen8_v0_0 (beta): Arbitary shape wave generator is included into this project
 *   QuadDec_SW_v0_1     : Quad decoder w/button swich. https://community.cypress.com/thread/30654
 *
 *   Other optional custom components used in the project:
 *   PSoC Annotation Library_v1_0: https://community.cypress.com/message/204321#204321
 *   StopWatch_v0_0 (beta) is included into the project 
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/



#include <project.h>
#include <stdio.h> // sprintf
//#include <math.h>
#include <DMA.h>
#include <RMS.h>


#define BlinkLED(); {Pin_LED_Write(1); CyDelayUs(50); Pin_LED_Write(0);} // blink LED indicator for debugging

char buff[64];      // UART output buffer            
 


//==============================================================================
// interrupt on DMA complete
//==============================================================================
volatile CYBIT flag_NrqReady = 0; 
CY_ISR(myNrqInt)                
{   
    flag_NrqReady = 1;
}



//============================================
// Forward declarations
//============================================
void WaveGen_UpdateWave();      // update output wave shape, amlitude, frequency
void PlotData();                // plot content of the RAM Buffer
void DisplayData();             // output to terminal


void Initialize(void)
{   
    CyGlobalIntEnable;          //enable global interrupts.
    
    UART_1_Start();             // Start UART  
    CyDelay(100);               // 
    
    StopWatch_1_Init();
    
    PGA_1_Start();              // Wave generator output buffer
    VDAC_1_Start();             // Wave generator DAC
    WaveGen_UpdateWave();       // update output wave shape, amlitude, frequency
    WaveGen_1_Start();          // start wave generator
    DDS_1_Start();              // start DDS
    Decoder_1_Start();          // start QuadDecoder for rotary encoder input
    
    DMA1_Config();              // initialize DMA for ADC-DMA-RAM transfer
    isrNrq_StartEx(myNrqInt);   //start isrNrq interrupt (DMA1 transfer complete)

    Opamp_RMS_Start();          // output RMS voltage to scope
    VDAC_RMS_Start();           // output RMS voltage to scope
    
    ConfigRMS16();              // configure weighting function buffer (16-bit precision)
    
    ADC_1_Start();              // initialize and enable ADC
    ADC_1_IRQ_Disable();        // disable the ADC ISR as it is not required
    ADC_1_StartConvert();       // start ADC sampling
}


int main()
{
    Initialize();

    for(;;)
    {   

        if (Decoder_1_PositionChanged)                  // encoder position changed
        { 
            BlinkLED();                                 // debugging...
            WaveGen_UpdateWave();                       // change amplitude        
            //DDS_1_SetFrequency(Decoder_1_Position);   // change wave frequency

        }
        
        if (Decoder_1_BtnPressed)                       // reset encoder position
        {    
            BlinkLED();                                 // debugging...
            Decoder_1_SetPosition(Decoder_1_StartPosition);   // reset position  
        }

        
        if (flag_NrqReady==1)                           // RAM buffer is populated -> "process" data
        {
            flag_NrqReady=0;                            // reset flag
            BlinkLED();                                 // completion indicator

            //StopWatch_1_Start();                      // for degugging
            GetRMS16();                                 // calculate RMS
            //StopWatch_1_Stop();
            
            VDAC_RMS_SetValue( RMSV_to_VDAC(RMS_V) );   // output control voltage to scope
            //PlotData();                               // read RAM buffer and send to PC for debugging
            DisplayData();                              // output result to terminal
        }
         
          
    }
    
} //main

   
//==============================================================================
// update output wave shape, amlitude, frequency
//==============================================================================

void WaveGen_UpdateWave()
{
    //WaveGen_1_CreateSineWave_C(125, 0);                 // update amplitude (sine centered at 128)
    //WaveGen_1_CreateRampWave_C(125, 0);                 // update amplitude (ramp centered at 128) //needs 8000 s/sec x 801 samples!
    WaveGen_1_CreateSineWave_C(Decoder_1_Position, 0);    // update amplitude (sine centered at 128)
    //WaveGen_1_CreateRampWave_C(Decoder_1_Position, 0);  // update amplitude (ramp centered at 128) //needs 8000 s/sec x 801 samples!
}


//==============================================================================
// 
//==============================================================================

void DisplayData(void) // output to terminal
{   
    sprintf(buff, "RMS: %.3f V\r\n", RMS_V);   
    //sprintf(buff, "RMS: %.4f V \t%lu\r\n", RMS_V, StopWatch_1_usec); // debugging 
    
    UART_1_PutString(buff);              // Send data to Terminal
}

/*
// for debugging->
//==============================================================================
// Send data to SerialPlot in Simple Binary format: int16 / uint16      
//==============================================================================
void PlotData_i16(int16 V)
{
    // 1-chan (16-bit) simple binary format:
    // Format:      Simple Binary
    // Channels:    1
    // Number Type: int16 / uint16
    // Endianness:  Little Endian
    //UART_1_PutChar(V);      // Send LSB first
    //UART_1_PutChar(V>>8); 
    
    uint8 * buff = (uint8 *) &V;    
    UART_1_PutArray (buff, 2);
 }

//==============================================================================
// Send data to SerialPlot charting s/w in Simple Binary format: int16 / uint16  
//   Serial PLOT v0.10.0
//   by Yavuz Ozderya
//   https://bitbucket.org/hyOzd/serialplot   
//==============================================================================
void PlotData()
{
    uint16 i;
    for (i=0; i<4*NO_SAMPLES; i++)
    {
        PlotData_i16(Buffer[i]);
    }
}
*/

/// END OF PROGRAM //////////////////////////////////////////////////////////////////////////////////////

